# Generated by roxygen2: do not edit by hand

S3method("[[",vectra_partition)
S3method(anti_join,vectra_node)
S3method(append_vtr,data.frame)
S3method(append_vtr,vectra_node)
S3method(arrange,vectra_node)
S3method(collect,data.frame)
S3method(collect,vectra_node)
S3method(collect,vectra_offload)
S3method(collect,vectra_shard)
S3method(collect_chunked,default)
S3method(collect_chunked,vectra_node)
S3method(collect_chunked,vectra_offload)
S3method(collect_chunked,vectra_partition)
S3method(collect_chunked,vectra_shard)
S3method(count,vectra_node)
S3method(cross_join,vectra_node)
S3method(dim,vectra_node)
S3method(distinct,vectra_node)
S3method(explain,vectra_node)
S3method(filter,vectra_node)
S3method(full_join,vectra_node)
S3method(fuzzy_join,vectra_node)
S3method(glimpse,vectra_node)
S3method(group_by,vectra_node)
S3method(group_map,vectra_partition)
S3method(group_modify,vectra_partition)
S3method(head,vectra_node)
S3method(inner_join,vectra_node)
S3method(interval_join,vectra_node)
S3method(kmer,vectra_node)
S3method(left_join,vectra_node)
S3method(lookup,vectra_schema)
S3method(materialize,vectra_node)
S3method(mutate,vectra_node)
S3method(print,offload_grade)
S3method(print,vectra_block)
S3method(print,vectra_grid)
S3method(print,vectra_network)
S3method(print,vectra_node)
S3method(print,vectra_partition)
S3method(print,vectra_raster)
S3method(print,vectra_schema)
S3method(print,vectra_shard)
S3method(pull,vectra_node)
S3method(reframe,vectra_node)
S3method(relocate,vectra_node)
S3method(rename,vectra_node)
S3method(resample,vectra_node)
S3method(right_join,vectra_node)
S3method(select,vectra_node)
S3method(semi_join,vectra_node)
S3method(sf::st_write,vectra_node)
S3method(slice,vectra_node)
S3method(slice_head,vectra_node)
S3method(slice_max,vectra_node)
S3method(slice_min,vectra_node)
S3method(slice_tail,vectra_node)
S3method(summarise,vectra_node)
S3method(tally,vectra_node)
S3method(transmute,vectra_node)
S3method(ungroup,vectra_node)
S3method(write_csv,data.frame)
S3method(write_csv,vectra_node)
S3method(write_sqlite,data.frame)
S3method(write_sqlite,vectra_node)
S3method(write_tiff,data.frame)
S3method(write_tiff,vectra_node)
S3method(write_vtr,data.frame)
S3method(write_vtr,vectra_node)
export(across)
export(anti_join)
export(append_vtr)
export(arrange)
export(as_embedding)
export(bind_cols)
export(bind_rows)
export(block_fuzzy_lookup)
export(block_lookup)
export(chunk_feeder)
export(collect)
export(collect_chunked)
export(collect_sf)
export(contours)
export(cosine)
export(count)
export(create_index)
export(cross_join)
export(delete_vtr)
export(desc)
export(diff_vtr)
export(distinct)
export(dot)
export(explain)
export(feature_knn)
export(filter)
export(floor_time)
export(focal)
export(full_join)
export(fuzzy_join)
export(glimpse)
export(grid)
export(group_by)
export(group_map)
export(group_modify)
export(has_index)
export(inner_join)
export(interval_join)
export(kmer)
export(l2)
export(left_join)
export(link)
export(lookup)
export(mask)
export(materialize)
export(mosaic)
export(mutate)
export(offload)
export(polygonize)
export(proximity)
export(pull)
export(rast_calc)
export(rast_feature_distance)
export(rasterize)
export(reframe)
export(relocate)
export(rename)
export(resample)
export(right_join)
export(roll_max)
export(roll_mean)
export(roll_min)
export(roll_n)
export(roll_sum)
export(select)
export(semi_join)
export(slice)
export(slice_head)
export(slice_max)
export(slice_min)
export(slice_tail)
export(spatial_centerline)
export(spatial_clip)
export(spatial_construct)
export(spatial_dissolve)
export(spatial_eliminate)
export(spatial_explode)
export(spatial_filter)
export(spatial_join)
export(spatial_knn)
export(spatial_line_merge)
export(spatial_locate)
export(spatial_map)
export(spatial_network)
export(spatial_overlay)
export(spatial_polygonize)
export(spatial_route)
export(spatial_service_area)
export(spatial_simplify)
export(spatial_smooth)
export(spatial_snap)
export(spatial_snap_grid)
export(spatial_split)
export(spatial_topology)
export(summarise)
export(summarize)
export(tally)
export(tbl)
export(tbl_bed)
export(tbl_csv)
export(tbl_fasta)
export(tbl_fastq)
export(tbl_sqlite)
export(tbl_tiff)
export(tbl_xlsx)
export(terrain)
export(tiff_band_names)
export(tiff_crs)
export(tiff_extract_points)
export(tiff_metadata)
export(transmute)
export(ungroup)
export(vec_build_overviews)
export(vec_close_raster)
export(vec_extract_points)
export(vec_open_raster)
export(vec_raster_layout)
export(vec_raster_times)
export(vec_read_pixel_series)
export(vec_read_time_slice)
export(vec_read_window)
export(vec_to_tiff)
export(vec_write_raster)
export(vec_write_time_cube)
export(vectra_mem)
export(vtr_schema)
export(warp)
export(write_csv)
export(write_sqlite)
export(write_tiff)
export(write_vtr)
export(zonal)
importFrom(libgeos,libgeos_version)
importFrom(rlang,as_function)
importFrom(rlang,enquo)
importFrom(rlang,expr)
importFrom(rlang,is_formula)
importFrom(stats,median)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(utils,head)
useDynLib(vectra, .registration = TRUE)
