% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_grouping.R
\name{group_by}
\alias{group_by}
\title{Group a vectra query by columns}
\usage{
group_by(.data, ...)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{...}{Grouping column names (unquoted).}
}
\value{
A \code{vectra_node} with grouping information stored.
}
\description{
Group a vectra query by columns
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> group_by(cyl) |> summarise(avg = mean(mpg)) |> collect()
unlink(f)

}
