% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/explain.R
\name{explain}
\alias{explain}
\title{Print the execution plan for a vectra query}
\usage{
explain(x, ...)
}
\arguments{
\item{x}{A \code{vectra_node} object.}

\item{...}{Ignored.}
}
\value{
Invisible \code{x}.
}
\description{
Shows the node types, column schemas, and structure of the lazy query plan.
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> filter(cyl > 4) |> select(mpg, cyl) |> explain()
unlink(f)

}
