% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_utility.R
\name{slice}
\alias{slice}
\title{Select rows by position}
\usage{
slice(.data, ...)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{...}{Integer row indices (positive or negative).}
}
\value{
A data.frame with the selected rows.
}
\description{
Select rows by position
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> slice(1, 3, 5)
unlink(f)

}
