% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_utility.R
\name{head.vectra_node}
\alias{head.vectra_node}
\title{Limit results to first n rows}
\usage{
\method{head}{vectra_node}(x, n = 6L, ...)
}
\arguments{
\item{x}{A \code{vectra_node} object.}

\item{n}{Number of rows to return.}

\item{...}{Ignored.}
}
\value{
A data.frame with the first \code{n} rows.
}
\description{
Limit results to first n rows
}
