% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_basic.R
\name{desc}
\alias{desc}
\title{Mark a column for descending sort order}
\usage{
desc(x)
}
\arguments{
\item{x}{A column name.}
}
\value{
A marker used by \code{\link[=arrange]{arrange()}}.
}
\description{
Used inside \code{\link[=arrange]{arrange()}} to sort a column in descending order.
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> arrange(desc(mpg)) |> collect() |> head()
unlink(f)

}
