% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_utility.R
\name{pull}
\alias{pull}
\title{Extract a single column as a vector}
\usage{
pull(.data, var = -1)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{var}{Column name (unquoted) or positive integer position.}
}
\value{
A vector.
}
\description{
Extract a single column as a vector
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> pull(mpg) |> head()
unlink(f)

}
