% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_utility.R
\name{relocate}
\alias{relocate}
\title{Relocate columns}
\usage{
relocate(.data, ..., .before = NULL, .after = NULL)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{...}{Column names to move.}

\item{.before}{Column name to place before (unquoted).}

\item{.after}{Column name to place after (unquoted).}
}
\value{
A new \code{vectra_node} with reordered columns.
}
\description{
Relocate columns
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> relocate(hp, wt, .before = cyl) |> collect() |> head()
unlink(f)

}
