% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/index.R
\name{has_index}
\alias{has_index}
\title{Check if a hash index exists for a .vtr column}
\usage{
has_index(path, column)
}
\arguments{
\item{path}{Path to a \code{.vtr} file.}

\item{column}{Character vector. Name(s) of column(s).}
}
\value{
Logical scalar: \code{TRUE} if a \code{.vtri} index file exists.
}
\description{
Check if a hash index exists for a .vtr column
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(data.frame(id = letters, val = 1:26, stringsAsFactors = FALSE), f)
has_index(f, "id")   # FALSE
create_index(f, "id")
has_index(f, "id")   # TRUE
unlink(c(f, paste0(f, ".id.vtri")))

}
