% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_utility.R
\name{transmute}
\alias{transmute}
\title{Keep only columns from mutate expressions}
\usage{
transmute(.data, ...)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{...}{Named expressions.}
}
\value{
A new \code{vectra_node} with only the computed columns.
}
\description{
Like \code{\link[=mutate]{mutate()}} but drops all other columns.
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> transmute(kpl = mpg * 0.425) |> collect() |> head()
unlink(f)

}
