% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_basic.R
\name{select}
\alias{select}
\title{Select columns from a vectra query}
\usage{
select(.data, ...)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{...}{Column names (unquoted).}
}
\value{
A new \code{vectra_node} with only the selected columns.
}
\description{
Select columns from a vectra query
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> select(mpg, cyl) |> collect() |> head()
unlink(f)

}
