% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.R
\name{print.vectra_node}
\alias{print.vectra_node}
\title{Print a vectra query node}
\usage{
\method{print}{vectra_node}(x, ...)
}
\arguments{
\item{x}{A \code{vectra_node} object.}

\item{...}{Ignored.}
}
\value{
Invisible \code{x}.
}
\description{
Print a vectra query node
}
