% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_grouping.R
\name{ungroup}
\alias{ungroup}
\title{Remove grouping from a vectra query}
\usage{
ungroup(x, ...)
}
\arguments{
\item{x}{A \code{vectra_node} object.}

\item{...}{Ignored.}
}
\value{
An ungrouped \code{vectra_node}.
}
\description{
Remove grouping from a vectra query
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> group_by(cyl) |> ungroup()
unlink(f)

}
