% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl.R
\name{tbl}
\alias{tbl}
\title{Create a lazy table reference from a .vtr file}
\usage{
tbl(path)
}
\arguments{
\item{path}{Path to a \code{.vtr} file.}
}
\value{
A \code{vectra_node} object representing a lazy scan of the file.
}
\description{
Opens a vectra1 file and returns a lazy query node. No data is read until
\code{\link[=collect]{collect()}} is called.
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
node <- tbl(f)
print(node)
unlink(f)

}
