% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/write.R
\name{write_sqlite}
\alias{write_sqlite}
\title{Write query results or a data.frame to a SQLite table}
\usage{
write_sqlite(x, path, table, ...)
}
\arguments{
\item{x}{A \code{vectra_node} (lazy query) or a \code{data.frame}.}

\item{path}{File path for the SQLite database.}

\item{table}{Name of the table to create/write into.}

\item{...}{Reserved for future use.}
}
\value{
Invisible \code{NULL}.
}
\description{
For \code{vectra_node} inputs, data is streamed batch-by-batch to disk without
materializing the full result in memory. For \code{data.frame} inputs, the data
is written directly.
}
\examples{
db <- tempfile(fileext = ".sqlite")
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars[1:5, ], f)
tbl(f) |> write_sqlite(db, "cars")
unlink(c(f, db))

}
