% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/verbs_utility.R
\name{rename}
\alias{rename}
\title{Rename columns}
\usage{
rename(.data, ...)
}
\arguments{
\item{.data}{A \code{vectra_node} object.}

\item{...}{Rename pairs: \code{new_name = old_name}.}
}
\value{
A new \code{vectra_node} with renamed columns.
}
\description{
Rename columns
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
tbl(f) |> rename(miles_per_gallon = mpg) |> collect() |> head()
unlink(f)

}
