% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raster.R
\name{vec_close_raster}
\alias{vec_close_raster}
\title{Close a .vec raster handle}
\usage{
vec_close_raster(r)
}
\arguments{
\item{r}{A \code{vectra_raster} returned by \code{vec_open_raster()}.}
}
\value{
Invisible \code{NULL}.
}
\description{
Idempotent. The handle is also auto-released by R's garbage collector.
}
