% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collect.R
\name{collect}
\alias{collect}
\title{Execute a lazy query and return a data.frame}
\usage{
collect(x, ...)
}
\arguments{
\item{x}{A \code{vectra_node} object.}

\item{...}{Ignored.}
}
\value{
A data.frame with the query results.
}
\description{
Pulls all batches from the execution plan and materializes the result as an
R data.frame.
}
\examples{
f <- tempfile(fileext = ".vtr")
write_vtr(mtcars, f)
result <- tbl(f) |> collect()
head(result)
unlink(f)

}
